/*
Module Name:

    ProtocolATM32.c


Revision History:

--*/

#include "stdafx.h"
#include <direct.h>
#include <winsock2.h>
#include "ntddpack.h"
#include "conpack.h"
#include "winsvc.h" 
#include "patmif.h"
#include "common.h"
 
typedef int NDIS_STATUS, *PNDIS_STATUS; // note default size

#define STATUS_NOT_SUPPORTED           (0xC00000BBL)
#define NDIS_STATUS_NOT_SUPPORTED		((NDIS_STATUS)STATUS_NOT_SUPPORTED)


#define STATE_NOT_INSTALLED  0
#define STATE_INSTALLED      1
#define STATE_STARTED        2

static int ProtocolATMDriverState = STATE_NOT_INSTALLED;
static CHAR szError[257];



//
//  FUNCTION: GetLastErrorText
//
//  PURPOSE: copies error message text to string
//
//  PARAMETERS:
//    lpszBuf - destination buffer
//    dwSize - size of buffer
//
//  RETURN VALUE:
//    destination buffer
//
//  COMMENTS:
//
static LPTSTR GetLastErrorText( LPTSTR lpszBuf, DWORD dwSize )
{
   DWORD dwRet;
   LPTSTR lpszTemp = NULL;

   dwRet = FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM |FORMAT_MESSAGE_ARGUMENT_ARRAY,
                          NULL,
                          GetLastError(),
                          LANG_NEUTRAL,
                          (LPTSTR)&lpszTemp,
                          0,
                          NULL );

   // supplied buffer is not long enough
   if ( !dwRet || ( (long)dwSize < (long)dwRet+14 ) )
      lpszBuf[0] = TEXT('\0');
   else
   {
      lpszTemp[lstrlen(lpszTemp)-2] = TEXT('\0');  //remove cr and newline character
      _stprintf( lpszBuf, TEXT("%s (0x%x)"), lpszTemp, GetLastError() );
   }

   if ( lpszTemp )
      LocalFree((HLOCAL) lpszTemp );

   return lpszBuf;
}

/*-------------------------------------------------------------------------------------

Routine Description:

    This rotine this routine allocates a ProtocolATM object for use
    in sending a receiveing ProtocolATMs

Arguments:

    Adapter object returned from ProtocolATMOpenAdapter

Return Value:

    SUCCESS - returns ProtocolATM object
    FAILURE - NULL

--*/


LPPROTOCOL_DRIVER ProtocolATMAllocateData( void )
{
	LPPROTOCOL_DRIVER lpProtocolATM;

	lpProtocolATM = (LPPROTOCOL_DRIVER) calloc(  1,sizeof(PROTOCOL_DRIVER));

	if ( lpProtocolATM == NULL ) 
	{

        THROW_MESSAGE("Error in function <malloc>",0);
		return NULL;
	}

	lpProtocolATM->OverLapped.hEvent = CreateEvent( NULL, FALSE, FALSE,  NULL );

	if ( lpProtocolATM->OverLapped.hEvent == NULL )
	{
        THROW_MESSAGE("Error in function <CreateEvent>",0);
		free(lpProtocolATM);
		return NULL;
	}

	lpProtocolATM->Buffer	  		= NULL;
	lpProtocolATM->Length	  		= 0;
	lpProtocolATM->ulBytesReceived	= 0;
	lpProtocolATM->bIoComplete		= FALSE;

	return lpProtocolATM;
}

/*-------------------------------------------------------------------------------------

Routine Description:

    This rotine initializes a ProtocolATM object to point to
    a memory buffer described by Start address and length

Arguments:

    lpProtocolATM   -  ProtocolATM object returned by ProtocolATMAllocateData

    Length     -  Length of memory buffer

Return Value:


--*/
BOOL ProtocolATMInitData( LPPROTOCOL_DRIVER	lpProtocolATM,  UINT Length )
{

	if ( lpProtocolATM->Buffer )
		free( lpProtocolATM->Buffer );

	lpProtocolATM->Buffer = (PVOID) calloc( 1,Length );

	if ( lpProtocolATM->Buffer ) 																
		lpProtocolATM->Length = Length;
	else
		lpProtocolATM->Length = 0;

	return (BOOL) (lpProtocolATM->Length != 0);
}

VOID  ProtocolATMFreeData( LPPROTOCOL_DRIVER lpProtocolATM )
{
	if (lpProtocolATM)
	{
		if ( lpProtocolATM->Buffer )
			free( lpProtocolATM->Buffer );

		CloseHandle(lpProtocolATM->OverLapped.hEvent);

		free(lpProtocolATM);
    }
}




/*++

Routine Description:

    This rotine sends an ioctl ProtocolATM to the adapter

Arguments:

    pAdapter  - AdapterObject return by ProtocolATMOpenAdapter

    lpProtocolATM         - ProtocolATM object returned by ProtocolATMAllocateData and initialized
                       by ProtocolATMInitData

    bSync            - TRUE if service should wait for ProtocolATM to transmit

Return Value:

    SUCCESS - TRUE if succeeded and SYNC==TRUE
    FAILURE -

--*/
BOOL ProtocolATMDeviceIoControl( PATM_ADAPTER pAdapter,LPPROTOCOL_DRIVER lpProtocolATM,
					   ULONG		ulIoctl,
					   BOOL		bSync )
{
	BOOL Result;
	DWORD dwError;

	_ASSERTE( lpProtocolATM != NULL );

	lpProtocolATM->OverLapped.Offset		= 0;
	lpProtocolATM->OverLapped.OffsetHigh	= 0;
	lpProtocolATM->ulBytesReceived		    = 0;

	if ( !ResetEvent( lpProtocolATM->OverLapped.hEvent ) ) 
	{
		lpProtocolATM->bIoComplete = FALSE;
		return FALSE;
	}

	Result = DeviceIoControl(	pAdapter->hFile,
								ulIoctl, 
								lpProtocolATM->Buffer, 
								lpProtocolATM->Length, 
								lpProtocolATM->Buffer, 
								lpProtocolATM->Length, 
								&(lpProtocolATM->ulBytesReceived), 
								&(lpProtocolATM->OverLapped) );

	if ( !Result && bSync )
	{
		//
		//  They want to wait
		//

		if ( (dwError = GetLastError()) == ERROR_IO_PENDING )
		{
			Result = GetOverlappedResult(	pAdapter->hFile,
											&(lpProtocolATM->OverLapped),
											&(lpProtocolATM->ulBytesReceived),
											TRUE );
		}
	} 

	lpProtocolATM->bIoComplete = Result;

	return Result;
}


    



static BOOL installNTProtocolATMDriver(void)
{
	SC_HANDLE SCManagerHandle=NULL;
	SC_HANDLE ServiceHandle =NULL;
	char DriverPath[_MAX_PATH+32];   /* Get the complete path filename: */
	int i;
	BOOL result = FALSE;


	ProtocolATMDriverState = STATE_NOT_INSTALLED;

    if (_getcwd((char *)DriverPath,_MAX_PATH) == NULL)
    {
        return FALSE;
    }
    if (DriverPath[strlen(DriverPath) -1] != '\\')
        strcat(DriverPath,"\\");
    
    strcat(DriverPath,PROTOCOL_DRIVER_NTX_DRIVERNAME);
    if (GetFileAttributes(DriverPath) == 0xffffffff)
	{

      THROW_MESSAGE("Error: Driver %s not found.",DriverPath);  
      return FALSE;

	}
	

	SCManagerHandle = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
	if (SCManagerHandle==NULL)
	{
        THROW_MESSAGE("Error: Function <OpenSCManager> returns error: %s",GetLastErrorText(szError,256));
		return FALSE;
	}

	for (i=0; i<5; i++)
	{
		ServiceHandle = CreateService( SCManagerHandle,
								   PROTOCOL_DRIVER_PREFIX , 
								   PROTOCOL_DRIVER_DESC,
								   SERVICE_ALL_ACCESS,
								   SERVICE_KERNEL_DRIVER,
								   SERVICE_DEMAND_START,
								   SERVICE_ERROR_NORMAL,
								   DriverPath,
								   NULL,
								   NULL,
								   NULL,
								   NULL, 
								   NULL);



		if (ServiceHandle == NULL)
		{
			if (GetLastError() == ERROR_SERVICE_EXISTS)
			{
				result = TRUE;
           		ProtocolATMDriverState |=STATE_INSTALLED;
				break;
			}
			else
			{
                THROW_MESSAGE("Error: Function <CreateService> returns error :%s",GetLastErrorText(szError,256));
				
				ServiceHandle = OpenService(SCManagerHandle, PROTOCOL_DRIVER_PREFIX,DELETE);
   	   
				if (ServiceHandle == NULL) 
				{
                    THROW_MESSAGE("Error: Function <OpenService> returns error: %s",GetLastErrorText(szError,256));
				}
   				else 
				{
					if (!DeleteService(ServiceHandle)) 
					{
                        THROW_MESSAGE("Error: Function <DeleteService> returns error: %s",GetLastErrorText(szError,256));
					}
					CloseServiceHandle(ServiceHandle);
   				}	
			}
		}
		else
		{
			result = TRUE;
       		ProtocolATMDriverState |=STATE_INSTALLED;
			break;
		}
		Sleep(500);

	}
	if (ServiceHandle != NULL)
	  CloseServiceHandle(ServiceHandle);

	if (SCManagerHandle!=NULL)
	  CloseServiceHandle(SCManagerHandle);
		
	return result;
}


BOOL RemoveNTProtocolATMDriver(void)
{

	SC_HANDLE       SCManagerHandle=NULL;
	SC_HANDLE       ServiceHandle =NULL;
	SERVICE_STATUS  ServiceStatus;
	
    BOOL result = FALSE;

   	SCManagerHandle = OpenSCManager(NULL, NULL, SC_MANAGER_CONNECT);
	
    if (SCManagerHandle==NULL)
	{
        THROW_MESSAGE("Error: Function <OpenSCManager> returns error: %s",GetLastErrorText(szError,256));
	}

	else
    {
        if (ProtocolATMDriverState & STATE_STARTED) 
	    {

   			ServiceHandle = OpenService(SCManagerHandle, PROTOCOL_DRIVER_PREFIX, SERVICE_STOP);
   	   
			if (ServiceHandle == NULL) 
			{
                THROW_MESSAGE("Error: Function <OpenService> for driver %s to stop fails",PROTOCOL_DRIVER_PREFIX);
			}
				
			else
			{ 
				if (!ControlService(ServiceHandle, SERVICE_CONTROL_STOP,&ServiceStatus)) 
				{
                    THROW_MESSAGE("Error: Function <ControlService> for driver %s to stop fails",PROTOCOL_DRIVER_PREFIX);
				}
   	   
				CloseServiceHandle(ServiceHandle);
			}
   	   
        }
        if (ProtocolATMDriverState & STATE_INSTALLED)
        { 
            ServiceHandle = OpenService(SCManagerHandle, PROTOCOL_DRIVER_PREFIX,DELETE);
   	   	    if (ServiceHandle == NULL) 
	        {
                THROW_MESSAGE("Error: Function <OpenService for driver %s to delete fails",PROTOCOL_DRIVER_PREFIX);
			}
   			else 
			{
				if (!DeleteService(ServiceHandle)) 
				{
                    THROW_MESSAGE("Error: Function <DeleteService for driver %s fails",PROTOCOL_DRIVER_PREFIX);
				}
				CloseServiceHandle(ServiceHandle);
   			}

		}

	    CloseServiceHandle(SCManagerHandle);
	}	  

    return result;
}

/*-------------------------------------------------------------------------------------

Routine Description:

    This routine opens an instance of the adapter

Arguments:

    lpstrAdapterName - unicode name of the adapter to open

Return Value:

    SUCCESS - returns a pointer to an adapter object
    FAILURE - NULL

--*/

BOOL ProtocolATMOpenAdapter( PATM_ADAPTER lpTheAdapter )
{
    TCHAR   SymbolicLink[ MAX_LINK_NAME_LENGTH ];
    TCHAR   DeviceName[MAX_LINK_NAME_LENGTH ];
	TCHAR   QueryResult[1024];
    BOOL    Status; 
											
															 
	lpTheAdapter->bCreatedDevice = FALSE;
	
		
	wsprintf(SymbolicLink, "%s",(LPSTR)(lpTheAdapter->IdentString));
	wsprintf(DeviceName, "\\Device\\%s%s",PROTOCOL_DRIVER_PREFIX,(LPSTR)(lpTheAdapter->IdentString));


	Status=	QueryDosDevice(SymbolicLink,QueryResult,sizeof(QueryResult));
			
	if (!Status) 
	{
		if ( (Status=GetLastError()) == ERROR_FILE_NOT_FOUND)
		{
			Status = DefineDosDevice(DDD_RAW_TARGET_PATH, SymbolicLink,DeviceName);
			lpTheAdapter->bCreatedDevice = TRUE;
		}
		else
		{
			Status=0;
	   }

	}
	if (Status)
	{
	
		wsprintf(SymbolicLink, TEXT("\\\\.\\%s%s"),PROTOCOL_DRIVER_PREFIX,(LPSTR)(lpTheAdapter->IdentString));
		lpTheAdapter->hFile = CreateFile(SymbolicLink,
										  GENERIC_WRITE | GENERIC_READ,
										  0,
										  NULL,
										  OPEN_EXISTING, 
										  FILE_FLAG_OVERLAPPED,
										  0);

		if (lpTheAdapter->hFile == INVALID_HANDLE_VALUE)
		{
		  
		   if (lpTheAdapter->bCreatedDevice)
		   {
				wsprintf(SymbolicLink, "%s",(LPSTR)(lpTheAdapter->IdentString));
				wsprintf(DeviceName, "\\Device\\%s%s",PROTOCOL_DRIVER_PREFIX,(LPSTR)(lpTheAdapter->IdentString));
				Status = DefineDosDevice(DDD_RAW_TARGET_PATH |DDD_REMOVE_DEFINITION, SymbolicLink,DeviceName);
				lpTheAdapter->bCreatedDevice = FALSE;
			}

            THROW_MESSAGE("Error: Fail to open ATM Adapter: %s",GetLastErrorText(szError,256));
			return  FALSE;
		}
        else
        	lpTheAdapter->bCreatedDevice = TRUE;

	}
	else
	{
	    THROW_MESSAGE("Error: Function <QueryDOSDevice> failed",0);
		return  FALSE;

	}

	return TRUE;
}


/*-------------------------------------------------------------------------------------

Routine Description:

    This routine closes a previouly opened adapter

Arguments:

    Adapter object returned from ProtocolATMOpenAdapter

Return Value:

--*/

VOID ProtocolATMCloseAdapter( PATM_ADAPTER lpTheAdapter )
{
    TCHAR   SymbolicLink[ MAX_LINK_NAME_LENGTH ];
    TCHAR   DeviceName[MAX_LINK_NAME_LENGTH ];
    BOOL    Status; 

	if(lpTheAdapter->hFile)
	{
		CloseHandle( lpTheAdapter->hFile );
		lpTheAdapter->hFile=NULL;
	}
	if (lpTheAdapter->bCreatedDevice)
	{
		wsprintf(SymbolicLink, "%s",(LPSTR)(lpTheAdapter->IdentString));
		wsprintf(DeviceName, "\\Device\\%s%s",PROTOCOL_DRIVER_PREFIX,(LPSTR)(lpTheAdapter->IdentString));
		Status = DefineDosDevice(DDD_RAW_TARGET_PATH |DDD_REMOVE_DEFINITION, SymbolicLink,DeviceName);
		lpTheAdapter->bCreatedDevice = FALSE;
	}

}






void ExtractIpAddresses(LPSTR IpString,int *pIpCount, DWORD *IpAddress)
{
	*pIpCount=0;

	int i=0;
	char *Name = IpString;
	IpString[strlen(IpString)+1]='\0';
	do 
	{
		if(IpString[i] == ',' || IpString[i] == ' ' || IpString[i]=='\0' )
		{
			IpString[i]='\0';
			IpAddress[*pIpCount]=inet_addr(Name);
	  		Name = &Name[strlen(Name)+1];
			(*pIpCount) ++;

		}
		i++;
	}while(*Name != '\0' );
}


char HostName[MAX_NAME_LENGTH];


static void FindTcpipSettings(int *pDeviceCount, struct Stack *pTcpStack)
{

	HKEY    EnumKey,EnumKey1,EnumKey2,EnumKey3;
	HKEY    TargetKey;
	HKEY    InterfaceKey;
	LONG    Status;
	int     i;
	unsigned long EnumLength;
	DWORD   RegType;
	char    *EnumStr;
    TCHAR   *EnumStr2;
	*pDeviceCount=0;

	EnumStr	=(char *)malloc(MAX_NAME_LENGTH+1);
	if (EnumStr==NULL)
		return;

	EnumStr2=(char *)malloc(MAX_NAME_LENGTH+1);
	if (EnumStr2 == NULL)
	{
		free(EnumStr);
		return;
	}

	Status = RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT("SYSTEM\\CurrentControlSet\\Control\\Network\\{4D36E972-E325-11CE-BFC1-08002BE10318}"), 0, KEY_READ, &EnumKey);
	if (Status == ERROR_SUCCESS)
	{		
        i=0;
     	EnumStr[0]='\0';
		EnumLength=MAX_NAME_LENGTH;

		while ((Status=RegEnumKeyEx(EnumKey, i++,(LPTSTR)EnumStr,&EnumLength, 0, 0, 0, 0))== ERROR_SUCCESS)
        {
    	    if (Status == ERROR_SUCCESS && EnumStr[0]== '{')
		    {
                strncpy(pTcpStack[*pDeviceCount].NetIdent,EnumStr,MAX_NAME_LENGTH);
			    Status = RegOpenKeyEx(EnumKey,pTcpStack[*pDeviceCount].NetIdent, 0, KEY_READ, &EnumKey2);
    	        if (Status == ERROR_SUCCESS)
		        {
			        Status = RegOpenKeyEx(EnumKey2,TEXT("Connection"), 0, KEY_READ, &EnumKey3);
			        RegCloseKey(EnumKey2);
			        if (Status == ERROR_SUCCESS)
			        {
				        EnumStr2[0]='\0';
				        EnumLength=MAX_NAME_LENGTH;
				        Status = RegQueryValueEx(EnumKey3, TEXT("PnpInstanceID"), NULL, &RegType,(LPBYTE)EnumStr2, &EnumLength);
				        if (Status == ERROR_SUCCESS)
				        {
					        RegCloseKey(EnumKey3);
	                        strncpy(pTcpStack[*pDeviceCount].PnpInstanceID,EnumStr2,MAX_NAME_LENGTH-1);

					        Status = RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT("SYSTEM\\CurrentControlSet\\Enum"), 0, KEY_READ, &EnumKey1);
					        if (Status == ERROR_SUCCESS)
					        {
						        Status = RegOpenKeyEx(EnumKey1,EnumStr2, 0, KEY_READ, &EnumKey2);
						        if (Status == ERROR_SUCCESS)
						        {
							        Status=RegOpenKeyEx(EnumKey2,TEXT("Control"), 0, KEY_READ, &EnumKey3);
							        if (Status == ERROR_SUCCESS)
							        {
								        EnumStr2[0]='\0';
								        EnumLength=MAX_NAME_LENGTH;
								        Status=RegQueryValueEx(EnumKey2, TEXT("FriendlyNameDesc"), NULL,&RegType, (LPBYTE)EnumStr2, (unsigned long *)&EnumLength);
								        if (Status == ERROR_SUCCESS)
								        {
									        strncpy(pTcpStack[*pDeviceCount].DeviceDesc,EnumStr2,MAX_NAME_LENGTH-1);
									        pTcpStack[*pDeviceCount].DeviceDesc[MAX_NAME_LENGTH-1]='\0';
								        }
								        else
								        {
									        EnumStr2[0]='\0';
									        EnumLength=MAX_NAME_LENGTH;
									        Status=RegQueryValueEx(EnumKey2, TEXT("DeviceDesc"), NULL,&RegType, (LPBYTE)EnumStr2, (unsigned long *)&EnumLength);
									        if (Status == ERROR_SUCCESS)
									        {
										        strncpy(pTcpStack[*pDeviceCount].DeviceDesc,EnumStr2,MAX_NAME_LENGTH-1);
										        pTcpStack[*pDeviceCount].DeviceDesc[MAX_NAME_LENGTH-1]='\0';
									        }
									        else
										        strcpy(pTcpStack[*pDeviceCount].DeviceDesc,"----------");
	
									        (*pDeviceCount) ++;
									        RegCloseKey(EnumKey3);
								        }
									
							        }
							        RegCloseKey(EnumKey2);
						        }
						        RegCloseKey(EnumKey1);
					        }
                        }
				    }
			    }
	        }
            EnumStr[0]='\0';
		    EnumLength=MAX_NAME_LENGTH;
			if (*pDeviceCount >= MAX_LOCAL_IP_NUMBER)
				break;	
        }
        RegCloseKey(EnumKey);
    
    }	
	
	
 	Status = RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT("SYSTEM\\CurrentControlSet\\Services"), 0, KEY_READ, &EnumKey);
	if (Status == ERROR_SUCCESS)
	{
		for (i=0;i<*pDeviceCount; i++)
		{
		    // windows 2000
 			Status = RegOpenKeyEx(EnumKey, TEXT("Tcpip\\Parameters\\Interfaces"), 0, KEY_READ, &TargetKey);
			if (Status == ERROR_SUCCESS)
			{
 				Status = RegOpenKeyEx(TargetKey, (LPSTR) pTcpStack[i].NetIdent, 0, KEY_READ, &InterfaceKey);
				RegCloseKey(TargetKey);
			}

			if (Status == ERROR_SUCCESS)
			{
				EnumLength=MAX_NAME_LENGTH;
				EnumStr[0]='\0';
				if ((Status=RegQueryValueEx(InterfaceKey, TEXT("IpAddress"), NULL,&RegType, (LPBYTE) EnumStr, (unsigned long *)&EnumLength)) == ERROR_SUCCESS)
					pTcpStack[i].IpAddress=inet_addr(EnumStr);
				else
	 			pTcpStack[i].IpAddress=0;

				
				EnumLength=MAX_NAME_LENGTH;
				EnumStr[0]='\0';
				if ((Status=RegQueryValueEx(InterfaceKey, TEXT("SubnetMask"), NULL,&RegType, (LPBYTE) EnumStr, (unsigned long *)&EnumLength)) == ERROR_SUCCESS)
					pTcpStack[i].IpMask=inet_addr(EnumStr);
				else
					pTcpStack[i].IpMask=0;

				EnumLength=MAX_NAME_LENGTH;
				EnumStr[0]='\0';
				if ((Status=RegQueryValueEx(InterfaceKey, TEXT("DefaultGateway"), NULL,&RegType, (LPBYTE) EnumStr, (unsigned long *)&EnumLength)) == ERROR_SUCCESS)
				{
				 	ExtractIpAddresses(EnumStr,&pTcpStack[i].GatewayCount,(DWORD *)pTcpStack[i].GatewayAddress);
				}
				else
					pTcpStack[i].GatewayCount=0;

				EnumLength=MAX_NAME_LENGTH;
				EnumStr[0]='\0';
				if ((Status=RegQueryValueEx(InterfaceKey, TEXT("MTU"), NULL,&RegType, (LPBYTE) EnumStr, (unsigned long *)&EnumLength)) == ERROR_SUCCESS)
					pTcpStack[i].MaxMTU=*(ULONG *)(EnumStr);
				else
					pTcpStack[i].MaxMTU=-1;

				pTcpStack[i].IsEthernetDevice=TRUE;

				RegCloseKey(InterfaceKey);
			}
		}
		RegCloseKey(EnumKey);
	}
	

	free(EnumStr);
	free(EnumStr2);
	return ;
}






BOOL ProtocolATMStop()
{
	SERVICE_STATUS Status;
	SC_HANDLE  SCManagerHandle=NULL;
  	SC_HANDLE  SCServiceHandle=NULL;

	SCManagerHandle = OpenSCManager(NULL, NULL, SC_MANAGER_CONNECT);
	if (SCManagerHandle==NULL)
	{
       THROW_MESSAGE("Error: Function <OpenSCManager> returns error: %s",GetLastErrorText(szError,256));
		return FALSE;

	}
	else
	{
		SCServiceHandle = OpenService(SCManagerHandle, PROTOCOL_DRIVER_PREFIX, SERVICE_STOP);
		if (SCServiceHandle == NULL)
		{
           THROW_MESSAGE("Error: Function <OpenService> for driver %s to start fails",PROTOCOL_DRIVER_PREFIX);
			CloseServiceHandle(SCManagerHandle);
			return FALSE;
		}
		ControlService(SCServiceHandle, SERVICE_CONTROL_STOP, &Status);
		CloseServiceHandle(SCServiceHandle);
		CloseServiceHandle(SCManagerHandle);
		ProtocolATMDriverState &= ~STATE_STARTED;
		
	}

    return RemoveNTProtocolATMDriver();

}


BOOL ProtocolATMStart()
{
	BOOL Status;
	SC_HANDLE  SCManagerHandle=NULL;
	SC_HANDLE  SCServiceHandle=NULL;


	if (installNTProtocolATMDriver() != TRUE)
	{
 		return FALSE;
	}

	SCManagerHandle = OpenSCManager(NULL, NULL, SC_MANAGER_CONNECT);
	if (SCManagerHandle==NULL)
	{
       THROW_MESSAGE("Error: Function <OpenSCManager> returns error: %s",GetLastErrorText(szError,256));
		return FALSE;

	}
	else
	{

		SCServiceHandle = OpenService(SCManagerHandle, PROTOCOL_DRIVER_PREFIX, SERVICE_START);
		if (SCServiceHandle == NULL)
		{
           THROW_MESSAGE("Error: Function <OpenService> for driver %s to start fails",PROTOCOL_DRIVER_PREFIX);
			CloseServiceHandle(SCManagerHandle);
			return FALSE;
		}
		Status = StartService(SCServiceHandle, 0, NULL);
		if (!Status)
		{
			if (GetLastError() != ERROR_SERVICE_ALREADY_RUNNING)
			{
			
               THROW_MESSAGE("Error: Function <StartService> fails",0);
				CloseServiceHandle(SCServiceHandle);
				CloseServiceHandle(SCManagerHandle);
				return FALSE;
			}
		}
		CloseServiceHandle(SCServiceHandle);
		CloseServiceHandle(SCManagerHandle);
		ProtocolATMDriverState |=STATE_STARTED;
		
	}

	return TRUE;

}




BOOL FindNetworkATMAdapters(PATM_ADAPTER Adapter ,DWORD *AdapterCount )
{
	int i,j,ReturnCode;

	struct Stack *pTcpStack;  

	int DeviceCount=0;

    ReturnCode = FALSE;
	pTcpStack=(struct Stack *)calloc(MAX_LOCAL_IP_NUMBER,sizeof(struct Stack));
	if (pTcpStack!= NULL)
	{

        FindTcpipSettings(&DeviceCount,pTcpStack);
		if (DeviceCount && ProtocolATMStart())
		{
			j = *AdapterCount;
			for ( i=0; i<DeviceCount && j<MAX_ATM_ADAPTERS; i++)
			{
				if (strstr((char *)pTcpStack[i].DeviceDesc,"PROSUM PROATM Adapter") != NULL)
				{
					strcpy(Adapter[j].IdentString,pTcpStack[i].NetIdent);
					strcpy(Adapter[j].DriverDesc,pTcpStack[i].DeviceDesc);
					strcpy(Adapter[j].PnpInstanceID,pTcpStack[i].PnpInstanceID);
					Adapter[j].IpAddress=pTcpStack[j].IpAddress;
					Adapter[j].IpMask=pTcpStack[j].IpMask;
					if (ProtocolATMOpenAdapter(&Adapter[j]))
					{
						ProtocolATMCloseAdapter(&Adapter[j]);
						Adapter[j].AdapterType =  NDIS_ADAPTER;
						ReturnCode =TRUE;
						j++;
					}
				}
			}
			ProtocolATMStop();
			*AdapterCount=j;
		}
	}
	if (pTcpStack != NULL)
			free(pTcpStack);


	return ReturnCode;

}

BOOL ProtocolATMSetLoopback(PATM_ADAPTER  pAdapter, BOOL bLoopback)
{
	LPPROTOCOL_DRIVER lpProtocolATM=NULL;
    ULONG RequestLength;
    BOOL Result = FALSE;

	if (!ProtocolATMOpenAdapter(pAdapter))
		return FALSE;

    if (pAdapter->bCreatedDevice)
    {
	    lpProtocolATM = ProtocolATMAllocateData();
        RequestLength = sizeof(PROTOCOL_OID_DATA) -1 + 4;

	    if ( lpProtocolATM && ProtocolATMInitData( lpProtocolATM,RequestLength) )
	    {

	        #define pOidData ((PPROTOCOL_OID_DATA)(lpProtocolATM->Buffer))


			pOidData->Oid    = OID_ATM_SET_LOOPBACK_MODE;

			*(ULONG *)(pOidData->Data) = bLoopback;
		    pOidData->Length = 4;

	        Result = ProtocolATMDeviceIoControl( pAdapter,
									             lpProtocolATM,
									             IOCTL_PROTOCOL_SET_OID,
									             TRUE );

	        if ( !Result)
	        {
                THROW_MESSAGE("Error in function <DeviceIoControl>",0);
            }
        
      
	    }
        else
        {
            THROW_MESSAGE("Error in function <ProtocolATMAllocateData>",0);

        }
        ProtocolATMFreeData(lpProtocolATM);
    }

	ProtocolATMCloseAdapter(pAdapter);

    return Result;



}
BOOL ProtocolATMGetInformation(PATM_ADAPTER  pAdapter)
{
	LPPROTOCOL_DRIVER lpProtocolATM=NULL;
    ULONG RequestLength;
    BOOL Result = FALSE;
    PATM_APPLICATION_INFO pInfo;

	if (!ProtocolATMOpenAdapter(pAdapter))
		return FALSE;

    if (pAdapter->bCreatedDevice)
    {
	    lpProtocolATM = ProtocolATMAllocateData();
        RequestLength = sizeof(PROTOCOL_OID_DATA) - 1 + sizeof(ATM_APPLICATION_INFO);

	    if ( lpProtocolATM && ProtocolATMInitData( lpProtocolATM,RequestLength) )
	    {

	        #define pOidData ((PPROTOCOL_OID_DATA)(lpProtocolATM->Buffer))

		    pOidData->Oid    = OID_ATM_APPLICATION_INFO;
		    pOidData->Length = sizeof(ATM_APPLICATION_INFO );

	        Result = ProtocolATMDeviceIoControl( pAdapter,
									             lpProtocolATM,
									             IOCTL_PROTOCOL_QUERY_OID,
									             TRUE );

	        if ( Result && (lpProtocolATM->ulBytesReceived >= RequestLength) )
	        {
		        pInfo  = (PATM_APPLICATION_INFO)pOidData->Data;
                pAdapter->MemorySize = pInfo->MemorySize;
                pAdapter->Speed      = pInfo->Speed;
                pAdapter->LinkStatus = pInfo->LinkStatus;
                memcpy(&pAdapter->MacAddress, pInfo->MacAddress,6);
                pAdapter->DriverVersion = pInfo->DriverVersion;
         	    Result = TRUE;
   
            }
            else
            {
                THROW_MESSAGE("Error in function <DeviceIoControl>",0);

            }
        
      
	    }
        else
        {
            THROW_MESSAGE("Error in function <ProtocolATMAllocateData>",0);

        }
        ProtocolATMFreeData(lpProtocolATM);
    }
	ProtocolATMCloseAdapter(pAdapter);

    return Result;

}






